@@ -80,14 +80,14 @@ QMimeData* UMLClipboard::copy(bool fromView/*=false*/)
     if (fromView) {
         m_type = clip4;
         UMLView *view = UMLApp::app()->currentView();
-        if (view == NULL) {
+        if (view == 0) {
             uError() << "UMLApp::app()->currentView() is NULL";
-            return NULL;
+            return 0;
         }
         UMLScene *scene = view->umlScene();
-        if (scene == NULL) {
+        if (scene == 0) {
             uError() << "currentView umlScene() is NULL";
-            return NULL;
+            return 0;
         }
         m_WidgetList = scene->selectedWidgetsExt();
         //if there is no selected widget then there is no copy action
@@ -127,9 +127,14 @@ QMimeData* UMLClipboard::copy(bool fromView/*=false*/)
         // in the Diagram
         if (m_type == clip2) {
             foreach (UMLView* view, m_ViewList) {
-                fillObjectListForWidgets(view->umlScene()->widgetList());
+                UMLScene *scene = view->umlScene();
+                if (scene == 0) {
+                    uError() << "currentView umlScene() is NULL";
+                    continue;
+                }
+                fillObjectListForWidgets(scene->widgetList());
 
-                AssociationWidgetList associations = view->umlScene()->associationList();
+                AssociationWidgetList associations = scene->associationList();
                 foreach (AssociationWidget* association, associations) {
                     if (association->umlObject() != 0) {
                         m_ObjectList.append(association->umlObject());
@@ -231,8 +236,8 @@ void UMLClipboard::addRelatedWidgets()
     UMLWidget *pWA =0, *pWB = 0;
 
     foreach (UMLWidget* widget, m_WidgetList) {
-        if (widget->baseType() == WidgetBase::wt_Message) {
-            MessageWidget * pMessage = static_cast<MessageWidget *>(widget);
+        if (widget->isMessageWidget()) {
+            MessageWidget * pMessage = widget->asMessageWidget();
             pWA = (UMLWidget*)pMessage->objectWidget(Uml::RoleType::A);
             pWB = (UMLWidget*)pMessage->objectWidget(Uml::RoleType::B);
             if (!relatedWidgets.contains(pWA))
@@ -462,7 +467,7 @@ bool UMLClipboard::pasteClip2(const QMimeData* data)
     }
 
     if (NoteWidget::s_pCurrentNote) {
-        NoteWidget::s_pCurrentNote = NULL;
+        NoteWidget::s_pCurrentNote = 0;
     } else {
         foreach (UMLView* pView, views) {
             if (!doc->addUMLView(pView)) {
@@ -600,9 +605,9 @@ bool UMLClipboard::pasteClip5(const QMimeData* data)
     if (!lvitem || !Model_Utils::typeIsClassifier(lvitem->type())) {
         return false;
     }
-    UMLClassifier *parent = dynamic_cast<UMLClassifier*>(lvitem->umlObject());
+    UMLClassifier *parent = lvitem->umlObject()->asUMLClassifier();
 
-    if (parent == NULL) {
+    if (parent == 0) {
         uError() << "parent is not a UMLClassifier";
         return false;
     }
@@ -632,7 +637,7 @@ bool UMLClipboard::pasteClip5(const QMimeData* data)
                     QString newName = parent->uniqChildName(UMLObject::ot_Attribute, obj->name());
                     obj->setName(newName);
                 }
-                UMLAttribute *att = static_cast<UMLAttribute*>(obj);
+                UMLAttribute *att = obj->asUMLAttribute();
                 if (parent->addAttribute(att, idchanges)) {
                     result = true;
                 } else {
@@ -643,7 +648,7 @@ bool UMLClipboard::pasteClip5(const QMimeData* data)
             }
         case UMLObject::ot_Operation :
             {
-                UMLOperation *op = static_cast<UMLOperation*>(obj);
+                UMLOperation *op = obj->asUMLOperation();
                 UMLOperation *exist = parent->checkOperationSignature(op->name(), op->getParmList());
                 if (exist) {
                     QString newName = parent->uniqChildName(UMLObject::ot_Operation, obj->name());
@@ -659,7 +664,7 @@ bool UMLClipboard::pasteClip5(const QMimeData* data)
             }
         case UMLObject::ot_Template:
             {
-                UMLTemplate* tp = static_cast<UMLTemplate*>(obj);
+                UMLTemplate* tp = obj->asUMLTemplate();
                 UMLTemplate* exist = parent->findTemplate(tp->name());
                 if (exist) {
                     QString newName = parent->uniqChildName(UMLObject::ot_Template, obj->name());
@@ -675,7 +680,7 @@ bool UMLClipboard::pasteClip5(const QMimeData* data)
             }
         case UMLObject::ot_EnumLiteral:
            {
-               UMLEnum* enumParent = dynamic_cast<UMLEnum*>(parent);
+               UMLEnum* enumParent = parent->asUMLEnum();
                // if parent is not a UMLEnum, bail out immediately;
                if (!enumParent) {
                    result = false;
@@ -688,7 +693,7 @@ bool UMLClipboard::pasteClip5(const QMimeData* data)
                    QString newName = enumParent->uniqChildName(UMLObject::ot_EnumLiteral, obj->name());
                    obj->setName(newName);
                }
-               UMLEnumLiteral* enl = static_cast<UMLEnumLiteral*>(obj);
+               UMLEnumLiteral* enl = obj->asUMLEnumLiteral();
 
                if (enumParent->addEnumLiteral(enl, idchanges)) {
                    result = true;
@@ -700,7 +705,7 @@ bool UMLClipboard::pasteClip5(const QMimeData* data)
            }
         case UMLObject::ot_EntityAttribute :
             {
-                UMLEntity* entityParent = dynamic_cast<UMLEntity*>(parent);
+                UMLEntity* entityParent = parent->asUMLEntity();
                 // if parent is not a UMLEntity, bail out immediately;
                 if (!entityParent) {
                     result = false;
@@ -712,7 +717,7 @@ bool UMLClipboard::pasteClip5(const QMimeData* data)
                     QString newName = entityParent->uniqChildName(UMLObject::ot_EntityAttribute, obj->name());
                     obj->setName(newName);
                 }
-                UMLEntityAttribute *att = static_cast<UMLEntityAttribute*>(obj);
+                UMLEntityAttribute *att = obj->asUMLEntityAttribute();
 
                 if (entityParent->addEntityAttribute(att, idchanges)) {
                     result = true;
