@@ -17,6 +17,7 @@
 #include "association.h"
 #include "classifier.h"
 #include "codegen_utils.h"
+#include "datatype.h"
 #include "debug_utils.h"
 #include "model_utils.h"
 #include "uml.h"
@@ -27,6 +28,7 @@
 #include "umlclassifierlistitemlist.h"
 #include "classifierlistitem.h"
 #include "codegenerationpolicy.h"
+#include "enumliteral.h"
 
 // qt includes
 #include <QFile>
@@ -395,8 +397,11 @@ void CppWriter::writeClassDecl(UMLClassifier *c, QTextStream &cpp)
         uint i = 0;
         cpp << "enum " << className_ << " {" << m_endl;
         foreach (UMLClassifierListItem* lit, litList) {
+            UMLEnumLiteral *el = static_cast<UMLEnumLiteral *>(lit);
             QString enumLiteral = cleanName(lit->name());
             cpp << indent() << enumLiteral;
+            if (!el->value().isEmpty())
+                cpp << " = " << el->value();
             if (++i < (uint)litList.count())
                 cpp << ",";
             cpp << m_endl;
@@ -446,6 +451,7 @@ void CppWriter::writeClassDecl(UMLClassifier *c, QTextStream &cpp)
 
     // PUBLIC attribs/methods
     cpp << "public:" << m_endl << m_endl; // print visibility decl.
+    writeDataTypes(c, Uml::Visibility::Public, cpp);
     // for public: constructors are first ops we print out
     if (!c->isInterface())
         writeConstructorDecls(cpp);
@@ -456,6 +462,7 @@ void CppWriter::writeClassDecl(UMLClassifier *c, QTextStream &cpp)
     // PROTECTED attribs/methods
     //
     cpp << "protected" << ":" << m_endl << m_endl; // print visibility decl.
+    writeDataTypes(c, Uml::Visibility::Protected, cpp);
     writeHeaderFieldDecl(c, Uml::Visibility::Protected, cpp);
     writeHeaderAccessorMethodDecl(c, Uml::Visibility::Protected, cpp);
     writeOperations(c, true, Uml::Visibility::Protected, cpp);
@@ -463,6 +470,7 @@ void CppWriter::writeClassDecl(UMLClassifier *c, QTextStream &cpp)
     // PRIVATE attribs/methods
     //
     cpp << "private" << ":" << m_endl << m_endl; // print visibility decl.
+    writeDataTypes(c, Uml::Visibility::Private, cpp);
     writeHeaderFieldDecl(c, Uml::Visibility::Private, cpp);
     writeHeaderAccessorMethodDecl(c, Uml::Visibility::Private, cpp);
     writeOperations(c, true, Uml::Visibility::Private, cpp);
@@ -526,6 +534,11 @@ void CppWriter::writeAttributeDecls (UMLClassifier *c, Uml::Visibility::Enum vis
 
             QString staticValue = at->isStatic() ? QLatin1String("static ") : QString();
             QString typeName = fixTypeName(at->getTypeName());
+            int i = typeName.indexOf(QLatin1Char('['));
+            if (i > -1) {
+                varName += typeName.mid(i);
+                typeName = typeName.left(i);
+            }
             if (!documentation.isEmpty())
                 writeComment(documentation, indent(), stream);
             stream << indent() << staticValue << typeName << " " << varName << ";" << m_endl;
@@ -921,6 +934,16 @@ void CppWriter::writeSingleAttributeAccessorMethods(
     QString className = fixTypeName(fieldClassName);
     QString fldName = Codegen_Utils::capitalizeFirstLetter(fieldName);
     QString indnt = indent();
+    QString varName = QLatin1String("new_var");
+    QString fullVarName = varName;
+
+    int i = className.indexOf(QLatin1Char('['));
+    bool isArrayType = false;
+    if (i > -1) {
+        fullVarName += className.mid(i);
+        className = className.left(i);
+        isArrayType = true;
+    }
 
     // set method
     if (change == Uml::Changeability::Changeable) {
@@ -928,7 +951,7 @@ void CppWriter::writeSingleAttributeAccessorMethods(
         stream << indnt << "void ";
         if (!isHeaderMethod)
             stream << className_ << "::";
-        stream << "set" << fldName << " (" << className << " new_var)";
+        stream << "set" << fldName << " (" << className << " " << fullVarName << ")";
 
         if (writeMethodBody) {
             stream << indnt << " {" << m_endl;
@@ -937,12 +960,18 @@ void CppWriter::writeSingleAttributeAccessorMethods(
             m_indentLevel--;
             if (isStatic)
                 stream << className_ << "::";
-            stream << fieldVarName << " = new_var;" << m_endl;
+            if (isArrayType)
+                stream << "*" << fieldVarName << " = *" << varName << ";" << m_endl;
+            else
+                stream << fieldVarName << " = " << fullVarName << ";" << m_endl;
             stream << indnt << "}" << m_endl;
         } else
             stream << ";" << m_endl;
     }
 
+    if (i)
+        className += QLatin1String("*");
+
     // get method
     writeDocumentation(QLatin1String("Get the value of ") + fieldVarName, description, policyExt()->getDocToolTag() + QLatin1String("return the value of ") + fieldVarName, stream);
     stream << indnt << className << " ";
@@ -1085,6 +1114,26 @@ void CppWriter::writeConstructorMethods(UMLClassifier * c, QTextStream &stream)
     writeBlankLine(stream);
 }
 
+/**
+ * Write all datatypes for a given class.
+ * @param c                the class for which we are generating code
+ * @param permitScope      what type of method to write (by Scope)
+ * @param cpp              the stream associated with the output file
+ */
+void CppWriter::writeDataTypes(UMLClassifier *c, Uml::Visibility::Enum permitScope, QTextStream &stream)
+{
+    foreach (UMLObject* o, c->containedObjects()) {
+        if (o->visibility() != permitScope)
+            continue;
+        if (!o->isUMLDatatype())
+            continue;
+        UMLDatatype *d = o->asUMLDatatype();
+        if (d && d->isReference() && d->originType()) {
+            stream << indent() << "typedef " << d->originType()->name() << " " << d->name() << ";" << m_endl;
+        }
+    }
+}
+
 /**
  * Replaces `string' with STRING_TYPENAME.
  */
@@ -1185,6 +1234,13 @@ void CppWriter::writeOperations(UMLClassifier *c, UMLOperationList &oplist, bool
             UMLAttribute* at = atlIt.next();
             QString typeName = fixTypeName(at->getTypeName());
             QString atName = cleanName(at->name());
+            QString atNameType = atName;
+
+            int i = typeName.indexOf(QLatin1Char('['));
+            if (i > -1) {
+                atNameType += typeName.mid(i);
+                typeName = typeName.left(i);
+            }
             str += typeName + QLatin1Char(' ') + atName;
             const QString initVal = at->getInitialValue();
             if (! initVal.isEmpty())
@@ -1233,15 +1289,15 @@ void CppWriter::writeOperations(UMLClassifier *c, UMLOperationList &oplist, bool
 void CppWriter::printAssociationIncludeDecl(UMLAssociationList list, Uml::ID::Type myId, QTextStream &stream)
 {
     foreach (UMLAssociation *a, list) {
-        UMLClassifier *current = NULL;
+        UMLClassifier *current = 0;
         bool isFirstClass = true;
 
         // only use OTHER classes (e.g. we don't need to write includes for ourselves!!
         // AND only IF the roleName is defined, otherwise, it is not meant to be noticed.
         if (a->getObjectId(Uml::RoleType::A) == myId && !a->getRoleName(Uml::RoleType::B).isEmpty()) {
-            current = dynamic_cast<UMLClassifier*>(a->getObject(Uml::RoleType::B));
+            current = a->getObject(Uml::RoleType::B)->asUMLClassifier();
         } else if (a->getObjectId(Uml::RoleType::B) == myId && !a->getRoleName(Uml::RoleType::A).isEmpty()) {
-            current = dynamic_cast<UMLClassifier*>(a->getObject(Uml::RoleType::A));
+            current = a->getObject(Uml::RoleType::A)->asUMLClassifier();
             isFirstClass = false;
         }
 
