@@ -17,6 +17,8 @@
 #include "uml.h"
 #include "umldoc.h"
 #include "umllistview.h"
+#include "datatype.h"
+#include "operation.h"
 #include "debug_utils.h"
 #include "ast_utils.h"
 #include "codeimpthread.h"
@@ -50,8 +52,8 @@ CppTree2Uml::~CppTree2Uml()
 void CppTree2Uml::clear()
 {
     m_currentScope.clear();
-    m_currentNamespace[0] = NULL;  // index 0 is reserved (always NULL)
-    m_currentClass[0] = NULL;  // index 0 is reserved (always NULL)
+    m_currentNamespace[0] = 0;  // index 0 is reserved (always 0)
+    m_currentClass[0] = 0;  // index 0 is reserved (always 0)
     m_nsCnt = 0;
     m_clsCnt = 0;
 
@@ -162,14 +164,14 @@ void CppTree2Uml::parseTypedef(TypedefAST* ast)
                         typeId == m_currentNamespace[m_nsCnt]->name())
                     inner = m_currentNamespace[m_nsCnt];
                 else
-                    inner = Import_Utils::createUMLObject(UMLObject::ot_Class, typeId,
+                    inner = Import_Utils::createUMLObject(UMLObject::ot_Class, type,
                                                           m_currentNamespace[m_nsCnt]);
                 UMLObject *typedefObj =
                  Import_Utils::createUMLObject(UMLObject::ot_Datatype, id,
                                                m_currentNamespace[m_nsCnt]);
-                UMLClassifier *dt = static_cast<UMLClassifier*>(typedefObj);
+                UMLDatatype *dt = typedefObj->asUMLDatatype();
                 dt->setIsReference();
-                dt->setOriginType(static_cast<UMLClassifier*>(inner));
+                dt->setOriginType(inner->asUMLClassifier());
             } else {
                 Import_Utils::createUMLObject(UMLObject::ot_Class, id,
                                                m_currentNamespace[m_nsCnt],
@@ -184,7 +186,7 @@ void CppTree2Uml::parseTypedef(TypedefAST* ast)
 void CppTree2Uml::parseTemplateDeclaration(TemplateDeclarationAST* ast)
 {
     TemplateParameterListAST* parmListAST = ast->templateParameterList();
-    if (parmListAST == NULL)
+    if (parmListAST == 0)
         return;
     QList<TemplateParameterAST*> parmList = parmListAST->templateParameterList();
     for (int i = 0; i < parmList.size(); ++i) {
@@ -195,7 +197,7 @@ void CppTree2Uml::parseTemplateDeclaration(TemplateDeclarationAST* ast)
             NameAST* nameNode = typeParmNode->name();
             if (nameNode) {
                 QString typeName = nameNode->unqualifiedName()->text();
-                Model_Utils::NameAndType nt(typeName, NULL);
+                Model_Utils::NameAndType nt(typeName, 0);
                 m_templateParams.append(nt);
             } else {
                 uError() << "nameNode is NULL";
@@ -205,7 +207,7 @@ void CppTree2Uml::parseTemplateDeclaration(TemplateDeclarationAST* ast)
         ParameterDeclarationAST* valueNode = tmplParmNode->typeValueParameter();
         if (valueNode) {
             TypeSpecifierAST* typeSpec = valueNode->typeSpec();
-            if (typeSpec == NULL) {
+            if (typeSpec == 0) {
                 uError() << "typeSpec is NULL";
                 continue;
             }
@@ -214,7 +216,7 @@ void CppTree2Uml::parseTemplateDeclaration(TemplateDeclarationAST* ast)
                                                           m_currentNamespace[m_nsCnt]);
             DeclaratorAST* declNode = valueNode->declarator();
             NameAST* nameNode = declNode->declaratorId();
-            if (nameNode == NULL) {
+            if (nameNode == 0) {
                 uError() << "CppTree2Uml::parseTemplateDeclaration(value):"
                          << " nameNode is NULL";
                 continue;
@@ -271,6 +273,8 @@ void CppTree2Uml::parseFunctionDefinition(FunctionDefinitionAST* ast)
 //:unused:    bool isInline = false;
     bool isConstructor = false;
 
+    bool isConstExpression = false;
+
     if (funSpec){
 //:unused:        QList<AST*> l = funSpec->nodeList();
 //:unused:        for (int i = 0; i < l.size(); ++i) {
@@ -286,6 +290,8 @@ void CppTree2Uml::parseFunctionDefinition(FunctionDefinitionAST* ast)
             QString text = l.at(i)->text();
             if (text == QLatin1String("friend")) isFriend = true;
             else if (text == QLatin1String("static")) isStatic = true;
+            else if (text == QLatin1String("constexpr"))
+                isConstExpression = true;
         }
     }
 
@@ -296,13 +302,16 @@ void CppTree2Uml::parseFunctionDefinition(FunctionDefinitionAST* ast)
     uDebug() << id;
 
     UMLClassifier *c = m_currentClass[m_clsCnt];
-    if (c == NULL) {
+    if (c == 0) {
         uDebug() << id << ": need a surrounding class.";
         return;
     }
 
     QString returnType = typeOfDeclaration(typeSpec, d);
     UMLOperation *m = Import_Utils::makeOperation(c, id);
+    if (isConstExpression)
+        m->setStereotype(QLatin1String("constexpr"));
+
     // if a class has no return type, it could be a constructor or
     // a destructor
     if (d && returnType.isEmpty() && id.indexOf(QLatin1Char('~')) == -1)
@@ -310,7 +319,7 @@ void CppTree2Uml::parseFunctionDefinition(FunctionDefinitionAST* ast)
 
     parseFunctionArguments(d, m);
     Import_Utils::insertMethod(c, m, m_currentAccess, returnType,
-                              isStatic, false /*isAbstract*/, isFriend, isConstructor, m_comment);
+                               isStatic, false /*isAbstract*/, isFriend, isConstructor, m_comment);
     m_comment = QString();
 
 /* For reference, Kdevelop does some more:
@@ -369,7 +378,7 @@ void CppTree2Uml::parseClassSpecifier(ClassSpecifierAST* ast)
                                           m_currentNamespace[m_nsCnt],
                                           ast->comment(), QString(), true);
 
-    UMLClassifier *klass = static_cast<UMLClassifier*>(o);
+    UMLClassifier *klass = o->asUMLClassifier();
     flushTemplateParams(klass);
     if (ast->baseClause())
         parseBaseClause(ast->baseClause(), klass);
@@ -393,6 +402,21 @@ void CppTree2Uml::parseClassSpecifier(ClassSpecifierAST* ast)
 
     m_currentScope.pop_back();
 
+    // check is class is an interface
+    bool isInterface = true;
+    foreach(UMLOperation *op, klass->getOpList()) {
+        if (!op->isDestructorOperation() && op->isAbstract() == false)
+            isInterface = false;
+    }
+
+    foreach(UMLAttribute *attr, klass->getAttributeList()) {
+        if (!(attr->isStatic() && attr->getTypeName().contains(QLatin1String("const"))))
+            isInterface = false;
+    }
+
+    if (isInterface)
+        klass->setBaseType(UMLObject::ot_Interface);
+
     m_currentAccess = oldAccess;
     m_inSlots = oldInSlots;
     m_inSignals = oldInSignals;
@@ -401,7 +425,7 @@ void CppTree2Uml::parseClassSpecifier(ClassSpecifierAST* ast)
 void CppTree2Uml::parseEnumSpecifier(EnumSpecifierAST* ast)
 {
     NameAST *nameNode = ast->name();
-    if (nameNode == NULL)
+    if (nameNode == 0)
         return;  // skip constants
     QString typeName = nameNode->unqualifiedName()->text().trimmed();
     if (typeName.isEmpty())
@@ -413,7 +437,11 @@ void CppTree2Uml::parseEnumSpecifier(EnumSpecifierAST* ast)
     QList<EnumeratorAST*> l = ast->enumeratorList();
     for (int i = 0; i < l.size(); ++i) {
         QString enumLiteral = l.at(i)->id()->text();
-        Import_Utils::addEnumLiteral((UMLEnum*)o, enumLiteral);
+        QString enumLiteralValue = QString();
+        if (l.at(i)->expr()) {
+            enumLiteralValue = l.at(i)->expr()->text();
+        }
+        Import_Utils::addEnumLiteral((UMLEnum*)o, enumLiteral, QString(), enumLiteralValue);
     }
 }
 
@@ -442,7 +470,7 @@ void CppTree2Uml::parseElaboratedTypeSpecifier(ElaboratedTypeSpecifierAST* typeS
         m_thread->emitAskQuestion("Soll nach CppTree2Uml::parseElaboratedTypeSpecifier weiter gemacht werden?");
     }
 #endif
-    flushTemplateParams(static_cast<UMLClassifier*>(o));
+    flushTemplateParams(o->asUMLClassifier());
 }
 
 void CppTree2Uml::parseDeclaration2(GroupAST* funSpec, GroupAST* storageSpec,
@@ -473,7 +501,7 @@ void CppTree2Uml::parseDeclaration2(GroupAST* funSpec, GroupAST* storageSpec,
     }
 
     UMLClassifier *c = m_currentClass[m_clsCnt];
-    if (c == NULL) {
+    if (c == 0) {
         uDebug() << id << ": need a surrounding class.";
         return;
     }
@@ -518,6 +546,7 @@ void CppTree2Uml::parseFunctionDeclaration(GroupAST* funSpec, GroupAST* storageS
 //:unused:    bool isInline = false;
     bool isPure = decl->initializer() != 0;
     bool isConstructor = false;
+    bool isConstExpression = false;
 
     if (funSpec){
 //:unused:        QList<AST*> l = funSpec->nodeList();
@@ -534,6 +563,8 @@ void CppTree2Uml::parseFunctionDeclaration(GroupAST* funSpec, GroupAST* storageS
             QString text = l.at(i)->text();
             if (text == QLatin1String("friend")) isFriend = true;
             else if (text == QLatin1String("static")) isStatic = true;
+            else if (text == QLatin1String("constexpr"))
+                isConstExpression = true;
         }
     }
 
@@ -541,13 +572,15 @@ void CppTree2Uml::parseFunctionDeclaration(GroupAST* funSpec, GroupAST* storageS
     QString id = d->declaratorId()->unqualifiedName()->text();
 
     UMLClassifier *c = m_currentClass[m_clsCnt];
-    if (c == NULL) {
+    if (c == 0) {
         uDebug() << id << ": need a surrounding class.";
         return;
     }
 
     QString returnType = typeOfDeclaration(typeSpec, d);
     UMLOperation *m = Import_Utils::makeOperation(c, id);
+    if (isConstExpression)
+        m->setStereotype(QLatin1String("constexpr"));
     // if a class has no return type, it could de a constructor or
     // a destructor
     if (d && returnType.isEmpty() && id.indexOf(QLatin1Char('~')) == -1)
@@ -556,6 +589,8 @@ void CppTree2Uml::parseFunctionDeclaration(GroupAST* funSpec, GroupAST* storageS
     parseFunctionArguments(d, m);
     Import_Utils::insertMethod(c, m, m_currentAccess, returnType,
                                isStatic, isPure, isFriend, isConstructor, m_comment);
+    if  (isPure)
+        c->setAbstract(true);
     m_comment = QString();
 }
 
@@ -598,6 +633,11 @@ QString CppTree2Uml::typeOfDeclaration(TypeSpecifierAST* typeSpec, DeclaratorAST
         text += ptrOpList.at(i)->text();
     }
 
+    QList<AST*> arrays = declarator->arrayDimensionList();
+    for(int i = 0; i < arrays.size(); ++i) {
+        text += arrays.at(i)->text().replace(QLatin1String(" "), QLatin1String(""));
+    }
+
     return text;
 }
 
@@ -607,7 +647,7 @@ void CppTree2Uml::parseBaseClause(BaseClauseAST * baseClause, UMLClassifier* kla
     for (int i = 0; i < l.size(); ++i) {
         BaseSpecifierAST* baseSpecifier = l.at(i);
 
-        if (baseSpecifier->name() == NULL) {
+        if (baseSpecifier->name() == 0) {
                 uDebug() << "baseSpecifier->name() is NULL";
                 continue;
         }
@@ -619,7 +659,7 @@ void CppTree2Uml::parseBaseClause(BaseClauseAST * baseClause, UMLClassifier* kla
                                                      m_currentNamespace[m_nsCnt],
                                                      baseSpecifier->comment());
         Import_Utils::putAtGlobalScope(false);
-        Import_Utils::createGeneralization(klass, static_cast<UMLClassifier*>(c));
+        Import_Utils::createGeneralization(klass, c->asUMLClassifier());
     }
 }
 
